@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -79,6 +79,8 @@ public final class ConfigurationImpl implements ConfigurationService {
 
     private static final String EXT = ".properties";
 
+    private Map<String, Properties> propertiesByFile = new HashMap<String, Properties>();
+
     private final File[] dirs;
 
     private static final class PropertyFileFilter implements FileFilter {
@@ -139,11 +141,11 @@ public final class ConfigurationImpl implements ConfigurationService {
             } else if (!dirs[i].isDirectory()) {
                 throw new IllegalArgumentException(MessageFormat.format("Not a directory: {0}", directories[i]));
             }
-            processDirectory(dirs[i], fileFilter, properties, propertiesFiles);
+            processDirectory(dirs[i], fileFilter, properties, propertiesFiles, propertiesByFile);
         }
     }
 
-    private static void processDirectory(final File dir, final FileFilter fileFilter, final Map<String, String> properties, final Map<String, String> propertiesFiles) {
+    private static void processDirectory(final File dir, final FileFilter fileFilter, final Map<String, String> properties, final Map<String, String> propertiesFiles, Map<String, Properties> propertiesByFile) {
         final File[] files = dir.listFiles(fileFilter);
         if (files == null) {
             LOG.info(MessageFormat.format("Can't read {0}. Skipping.", dir));
@@ -151,19 +153,17 @@ public final class ConfigurationImpl implements ConfigurationService {
         }
         for (int i = 0; i < files.length; i++) {
             if (files[i].isDirectory()) {
-                processDirectory(files[i], fileFilter, properties, propertiesFiles);
+                processDirectory(files[i], fileFilter, properties, propertiesFiles, propertiesByFile);
             } else {
-                processPropertiesFile(files[i], properties, propertiesFiles);
+                processPropertiesFile(files[i], properties, propertiesFiles, propertiesByFile);
             }
         }
     }
 
-    private static void processPropertiesFile(final File propFile, final Map<String, String> properties, final Map<String, String> propertiesFiles) {
-        FileInputStream fis = null;
+    private static void processPropertiesFile(final File propFile, final Map<String, String> properties, final Map<String, String> propertiesFiles, Map<String, Properties> propertiesByFile) {
         try {
-            fis = new FileInputStream(propFile);
-            final Properties tmp = new Properties();
-            tmp.load(fis);
+            final Properties tmp = loadProperties(propFile);
+            propertiesByFile.put(propFile.getPath(), tmp);
             final int size = tmp.size();
             final Iterator<Entry<Object, Object>> iter = tmp.entrySet().iterator();
             for (int i = 0; i < size; i++) {
@@ -186,13 +186,20 @@ public final class ConfigurationImpl implements ConfigurationService {
             LOG.error(e.getMessage(), e);
         } catch (final IOException e) {
             LOG.error(e.getMessage(), e);
+        }
+    }
+
+    private static Properties loadProperties(final File propFile) throws FileNotFoundException, IOException {
+        final FileInputStream fis = new FileInputStream(propFile);
+        try {
+            final Properties tmp = new Properties();
+            tmp.load(fis);
+            return tmp;
         } finally {
-            if (null != fis) {
-                try {
-                    fis.close();
-                } catch (final IOException e) {
-                    LOG.error(e.getMessage(), e);
-                }
+            try {
+                fis.close();
+            } catch (final IOException e) {
+                LOG.error(e.getMessage(), e);
             }
         }
     }
@@ -262,18 +269,29 @@ public final class ConfigurationImpl implements ConfigurationService {
      * {@inheritDoc}
      */
     public Properties getFile(final String filename, final PropertyListener listener) {
+        String key = null;
+        for (String k : propertiesByFile.keySet()) {
+            if(k.endsWith(filename)) {
+                key = k;
+                break;
+            }
+        }
+        
+        if(key == null) {
+            return new Properties();
+        }
+        
+        Properties tmp = propertiesByFile.get(key);
         final Properties retval = new Properties();
-        final Iterator<Entry<String, String>> iter = propertiesFiles.entrySet().iterator();
-        while (iter.hasNext()) {
-            final Entry<String, String> entry = iter.next();
-            if (entry.getValue().endsWith(filename)) {
-                final String value;
-                if (null == listener) {
-                    value = getProperty(entry.getKey());
-                } else {
-                    value = getProperty(entry.getKey(), listener);
-                }
-                retval.put(entry.getKey(), value);
+        
+        for( Entry<Object, Object> entry : tmp.entrySet()) {
+            retval.put(entry.getKey(), entry.getValue());
+        }
+        
+        
+        if(listener != null) {
+            for(Object k : retval.keySet()) {
+                getProperty((String) k, listener);
             }
         }
         return retval;
